/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.nephos3d.resources.textures;

import com.jogamp.common.nio.Buffers;
import com.jogamp.opengl.GL3bc;
import java.nio.IntBuffer;
import fr.amap.lidar.amapvox.nephos3d.resources.Resource;
import net.jcip.annotations.GuardedBy;
import net.jcip.annotations.ThreadSafe;

@ThreadSafe
public abstract class Texture extends Resource {
        
    @GuardedBy("this")
    private final IntBuffer tbo;
    
    @GuardedBy("this")
    private TextureFormat format;
    
    @GuardedBy("this")
    private boolean generateMipmaps;
    
    private final int width;
    
    private final int height;
    
    public Texture(final String identifier, final int width, final int height,
            final TextureFormat format, final boolean generateMipmaps) {
        super(identifier);
        this.tbo = Buffers.newDirectIntBuffer(1);
        this.width = width;
        this.height = height;
        this.format = format;
        this.generateMipmaps = generateMipmaps;
    }

    public final synchronized int getIndex() {
        return tbo.get(0);
    }

    public final synchronized int getWidth() {
        return width;
    }

    public final synchronized int getHeight() {
        return height;
    }

    public final synchronized boolean isGenerateMipmaps() {
        return generateMipmaps;
    }
    
    protected final synchronized int getInternalFormat() {
        if (format != null) switch(format) {
            case RGB: return GL3bc.GL_RGB;
            case RGBA: return GL3bc.GL_RGBA;
            case sRGB: return GL3bc.GL_SRGB;
            case sRGBA: return GL3bc.GL_SRGB_ALPHA;
        }
        
        throw(new NullPointerException("The texture format is undefined"));
    }
    
    protected final synchronized int getExternalFormat() {
        if (format != null) switch(format) {
            case RGB: return GL3bc.GL_RGB;
            case RGBA: return GL3bc.GL_RGBA;
            case sRGB: return GL3bc.GL_RGB;
            case sRGBA: return GL3bc.GL_RGBA;
        }
        
        throw(new NullPointerException("The texture format is undefined"));
    }

    public final synchronized TextureFormat getFormat() {
        return format;
    }

    @Override
    protected void onLoad(final GL3bc gl) {
        gl.glGenTextures(1, tbo);
    }

    @Override
    protected void onUnload(final GL3bc gl) {
        gl.glDeleteTextures(1, tbo);
    }

}
