/* Input area */
in vec4 mainColor;

/* Uniform area */
uniform sampler2D albedo;
uniform samplerCube cubemap;

/* Output color */
out vec4 color;

void main(void)
{
    if(mainColor.a < 0.1)
    {
        discard;
    }

    vec3 normal = normalize(fragNormal);
    vec3 view = normalize(viewPos - fragPosition);

    float NdotV = max(dot(normal, view), 0.0);
    vec3 refl = reflect(view, normal);

    float fresnel = pow(1 - NdotV, 5);

    vec4 albedoColor = texture(albedo, fragTexcoord);
    vec4 cubemapColor = textureLod(cubemap, refl, 2);

    albedoColor = albedoColor.rgb == vec3(0) ? vec4(1) : albedoColor;

    float near = 0.01;
    float far = 1000;
    float depthAtten = ((2.0 * near * far) / (far + near - 
            (gl_FragCoord.z * 2.0 - 1.0) * (far - near))) / far;

    /* Set the output color */
    if (cubemapColor.rgb == vec3(0))
    {
        color = vec4(mainColor.rgb * albedoColor.rgb * (1 - depthAtten), 1);
    }
    else
    {
        color = vec4((mainColor.rgb * albedoColor.rgb + cubemapColor.rgb * fresnel) * (1 - depthAtten), 1);
    }
}